<?php

namespace Drupal\content_moderation_edit_notify\Controller;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\node\NodeInterface;
use Drupal\Core\Language\LanguageInterface;

/**
 * Returns responses for Moderation Notify module routes.
 */
class ContentModerationNotifyController extends ControllerBase {

  /**
   * Check if a new revision exist for a current node.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request of the page.
   * @param \Drupal\node\NodeInterface $node
   *   The node that need to be checked.
   * @param int $vid
   *   Current vid to check for.
   *
   * @return Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function checkNewRevision(Request $request, NodeInterface $node, $vid) {
    $response = NULL;

    // Get last revision for this node.
    $node_storage = \Drupal::service('entity_type.manager')->getStorage('node');
    $latest_revision_vid = $node_storage->getLatestRevisionId($node->id());
    $latest_revision = $node_storage->loadRevision($latest_revision_vid);

    // Check if revision translation is affected. If not, no message necessary.
    $lang_code = \Drupal::languageManager()->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
    if ($node->hasTranslation($lang_code)) {
      if (!$latest_revision->getTranslation($lang_code)->isRevisionTranslationAffected()) {
        return new JsonResponse($response);
      }
    }

    // Check if current user is editing the last revision.
    if ($latest_revision_vid != $vid) {
      $response['last_vid'] = $latest_revision_vid;
      // Get current settings to build the message.
      $config = $this->config('content_moderation_edit_notify.settings');

      $token_service = \Drupal::token();

      // A "live" entity revision is one whose latest revision is also the
      // default, and whose moderation state, if any, is a published state.
      $moderation_info = \Drupal::service('content_moderation.moderation_information');
      if ($moderation_info->isLiveRevision($latest_revision)) {
        $message = $config->get('message_published');
        $type = "error";
      }
      else {
        $message = $config->get('message_unpublished');
        $type = "warning";
      }
      $message = $token_service->replace($message, ['node' => $latest_revision]);

      // Mimic status messages.
      $status_message = [
        '#markup' => $message,
        '#prefix' => '<div class="content-moderation-notify messages messages--' . $type . ' notify-' . $latest_revision_vid . '">',
        '#suffix' => '</div>',
      ];
      $response['message'] = \Drupal::service('renderer')->render($status_message);
    }

    return new JsonResponse($response);
  }

}
